@@ -25,6 +25,10 @@ Utilities for working with :pep:`508` requirements.
 #  OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 #  OR OTHER DEALINGS IN THE SOFTWARE.
 #
+#  check_dependencies based on https://stackoverflow.com/a/29044693/3092681
+#  Copyright © 2015 TehTechGuy
+#  Licensed under CC-BY-SA
+#
 
 # stdlib
 import warnings
@@ -32,6 +36,7 @@ from abc import ABC
 from typing import Any, Callable, Dict, Iterable, Iterator, List, Optional, Set, Tuple, Union, overload
 
 # 3rd party
+import dom_toml
 from domdf_python_tools.compat import importlib_metadata
 from domdf_python_tools.doctools import prettify_docstrings
 from domdf_python_tools.iterative import natmax, natmin
@@ -54,6 +59,9 @@ __all__ = [
 		"RequirementsManager",
 		"list_requirements",
 		"check_dependencies",
+		"parse_requirements",
+		"parse_pyproject_dependencies",
+		"parse_pyproject_extras",
 		]
 
 operator_symbols = ("<=", '<', "!=", "==", ">=", '>', "~=", "===")
@@ -317,8 +325,7 @@ def read_requirements(
 		include_invalid: Literal[True],
 		*,
 		normalize_func: Callable[[str], str] = ...
-		) -> _read_requirements_ret_invalid:
-	...  # pragma: no cover
+		) -> _read_requirements_ret_invalid: ...
 
 
 @overload
@@ -327,8 +334,7 @@ def read_requirements(
 		include_invalid: Literal[False] = ...,
 		*,
 		normalize_func: Callable[[str], str] = ...
-		) -> _read_requirements_ret_valid:
-	...  # pragma: no cover
+		) -> _read_requirements_ret_valid: ...
 
 
 def read_requirements(
@@ -350,28 +356,73 @@ def read_requirements(
 	:return: The requirements, and a list of commented lines.
 	"""
 
+	requirements = PathPlus(req_file).read_lines()
+
+	if include_invalid:
+		return parse_requirements(requirements, include_invalid=True, normalize_func=normalize_func)
+
+	else:
+		return parse_requirements(requirements, include_invalid=False, normalize_func=normalize_func)
+
+
+@overload
+def parse_requirements(
+		requirements: Iterable[str],
+		*,
+		include_invalid: Literal[True],
+		normalize_func: Callable[[str], str] = ...
+		) -> _read_requirements_ret_invalid: ...
+
+
+@overload
+def parse_requirements(
+		requirements: Iterable[str],
+		*,
+		include_invalid: Literal[False] = ...,
+		normalize_func: Callable[[str], str] = ...
+		) -> _read_requirements_ret_valid: ...
+
+
+def parse_requirements(
+		requirements: Iterable[str],
+		*,
+		include_invalid: bool = False,
+		normalize_func: Callable[[str], str] = normalize
+		) -> _read_requirements_ret:
+	"""
+	Parse the given strings as :pep:`508` requirements.
+
+	.. versionadded:: 0.10.0
+
+	:param requirements:
+	:param include_invalid: If :py:obj:`True`, include invalid lines as the third element of the tuple.
+	:param normalize_func: Function to use to normalize the names of requirements.
+
+	:return: The requirements, and a list of commented lines.
+	"""
+
 	comments = []
 	invalid_lines: List[str] = []
-	requirements: Set[ComparableRequirement] = set()
+	parsed_requirements: Set[ComparableRequirement] = set()
 
-	for line in PathPlus(req_file).read_lines():
+	for line in requirements:
 		if line.startswith('#'):
 			comments.append(line)
 		elif line:
 			try:
 				req = ComparableRequirement(line)
 				req.name = normalize_func(req.name)
-				requirements.add(req)
+				parsed_requirements.add(req)
 			except InvalidRequirement:
 				invalid_lines.append(line)
 
 	if include_invalid:
-		return requirements, comments, invalid_lines
+		return parsed_requirements, comments, invalid_lines
 	else:
 		for line in invalid_lines:
 			warnings.warn(f"Ignored invalid requirement {line!r}")
 
-		return requirements, comments
+		return parsed_requirements, comments
 
 
 class RequirementsManager(ABC):
@@ -386,6 +437,8 @@ class RequirementsManager(ABC):
 	#. :meth:`~.write_requirements`
 
 	:param repo_path: Path to the repository root.
+
+	.. latex:vspace:: 20px
 	"""
 
 	target_requirements: Set[ComparableRequirement]
@@ -463,7 +516,7 @@ class RequirementsManager(ABC):
 
 	def remove_library_requirements(self) -> None:
 		"""
-		Remove requirements given in the library requirements.txt file.
+		Remove requirements given in the library's ``requirements.txt`` file.
 
 		This method may not return anything.
 		"""
@@ -536,8 +589,12 @@ def list_requirements(name: str, depth: int = 1) -> Iterator[Union[str, List[str
 			<requirement b>,
 		]
 
+	.. latex:clearpage::
+
 	:param name:
 	:param depth:
+
+	.. versionchanged:: 0.8.2  The requirements are now sorted alphabetically.
 	"""
 
 	req = ComparableRequirement(name)
@@ -547,7 +604,7 @@ def list_requirements(name: str, depth: int = 1) -> Iterator[Union[str, List[str
 	except importlib_metadata.PackageNotFoundError:
 		return
 
-	for requirement in [ComparableRequirement(r) for r in raw_deps]:
+	for requirement in [ComparableRequirement(r) for r in sorted(raw_deps)]:
 		if requirement.marker:
 			if req.extras:
 				extras = list(req.extras)[0]
@@ -596,3 +653,87 @@ def check_dependencies(dependencies: Iterable[str], prt: bool = True) -> List[st
 		print('')
 
 	return missing_modules
+
+
+def parse_pyproject_dependencies(
+		pyproject_file: PathLike,
+		flavour: Literal["pep621", "flit", "auto"] = "auto",
+		*,
+		normalize_func: Callable[[str], str] = normalize
+		) -> Set[ComparableRequirement]:
+	"""
+	Parse the project's dependencies from its ``pyproject.toml`` file.
+
+	.. versionadded:: 0.10.0
+
+	:param pyproject_file:
+	:param flavour: Either ``'pep621'`` to parse from the :pep:`621` ``dependencies`` table,
+		or ``'flit'`` to parse the ``requires`` key in ``tool.flit.metadata`.
+		The string ``'auto`` will use ``'pep621'`` if available, otherwise try ``'flit'``.
+	:param normalize_func: Function to use to normalize the names of dependencies.
+
+	If no dependencies are defined an empty set is returned.
+	"""
+
+	config = dom_toml.load(pyproject_file)
+
+	dependencies = []
+
+	if flavour == "auto":
+		if "project" in config:
+			flavour = "pep621"
+		elif "flit" in config.get("tool", {}):
+			flavour = "flit"
+
+	if flavour == "pep621":
+		dependencies = config.get("project", {}).get("dependencies", [])
+	elif flavour == "flit":
+		dependencies = config.get("tool", {}).get("flit", {}).get("metadata", {}).get("requires", [])
+
+	return parse_requirements(dependencies, include_invalid=True, normalize_func=normalize_func)[0]
+
+
+def parse_pyproject_extras(
+		pyproject_file: PathLike,
+		flavour: Literal["pep621", "flit", "auto"] = "auto",
+		*,
+		normalize_func: Callable[[str], str] = normalize
+		) -> Dict[str, Set[ComparableRequirement]]:
+	"""
+	Parse the project's extra dependencies from its ``pyproject.toml`` file.
+
+	.. versionadded:: 0.10.0
+
+	:param pyproject_file:
+	:param flavour: Either ``'pep621'`` to parse from the :pep:`621` ``dependencies`` table,
+		or ``'flit'`` to parse the ``requires-extra`` key in ``tool.flit.metadata`.
+		The string ``'auto`` will use ``'pep621'`` if available, otherwise try ``'flit'``.
+	:param normalize_func: Function to use to normalize the names of dependencies.
+
+	If no extra dependencies are defined an empty dictionary is returned.
+
+	:rtype:
+
+	.. latex:clearpage::
+	"""
+
+	config = dom_toml.load(pyproject_file)
+
+	dependencies = {}
+
+	if flavour == "auto":
+		if "project" in config:
+			flavour = "pep621"
+		elif "flit" in config.get("tool", {}):
+			flavour = "flit"
+
+	if flavour == "pep621":
+		dependencies = config.get("project", {}).get("optional-dependencies", {})
+	elif flavour == "flit":
+		dependencies = config.get("tool", {}).get("flit", {}).get("metadata", {}).get("requires-extra", {})
+
+	return {
+			k: parse_requirements(v, include_invalid=True, normalize_func=normalize_func)[0]
+			for k,
+			v in dependencies.items()
+			}
